@@ -5,7 +5,7 @@ use Data::Printer::Object;
 use Data::Printer::Common;
 use Data::Printer::Config;
 
-our $VERSION = '0.99_022';
+our $VERSION = '0.99_023';
 
 my $rc_arguments;
 my %arguments_for;
@@ -23,6 +23,7 @@ sub import {
     if (@_ > 0) {
         $args = @_ == 1 ? shift : {@_};
         Data::Printer::Common::_warn(
+            undef,
             'Data::Printer can receive either a hash or a hash reference'
         ) unless ref $args eq 'HASH';
     }
@@ -182,28 +183,31 @@ sub _fetch_args_with {
 
     my $args_to_use = {};
     if (keys %$rc_arguments) {
-        $args_to_use = Data::Printer::Common::_merge_options(
+        $args_to_use = Data::Printer::Config::_merge_options(
             $args_to_use, $rc_arguments->{'_'}
         );
         if (exists $rc_arguments->{$caller}) {
-            $args_to_use = Data::Printer::Common::_merge_options(
+            $args_to_use = Data::Printer::Config::_merge_options(
                 $args_to_use, $rc_arguments->{$caller}
             );
         }
     }
     if ($arguments_for{$caller}) {
-        $args_to_use = Data::Printer::Common::_merge_options(
+        $args_to_use = Data::Printer::Config::_merge_options(
             $args_to_use, $arguments_for{$caller}
         );
     }
     if (keys %$run_properties) {
-        $args_to_use = Data::Printer::Common::_merge_options(
+        $run_properties = Data::Printer::Config::_expand_profile($run_properties)
+            if exists $run_properties->{profile};
+        $args_to_use = Data::Printer::Config::_merge_options(
             $args_to_use, $run_properties
         );
     }
     return $args_to_use;
 }
 
+
 sub _find_option {
     my ($key, $args, $caller, $default) = @_;
 
@@ -255,7 +259,7 @@ Want to see what's inside a variable in a complete, colored and human-friendly w
     &p( [ $one, $two, $three ] );
     &p( { foo => $foo, bar => $bar } );
 
-    # use postderef on perl 5.24 or later:
+    # or use postderef on perl 5.24 or later:
     p [ $one, $two, $three ]->@*;
     p { foo => $foo, bar => $bar }->%*;
 
@@ -273,7 +277,7 @@ The C<np()> function is the same as C<p()> but will return the string
 containing the dump. By default it has no colors, but you can change that
 easily too.
 
-That's it :)
+That's pretty much it :)
 
 Data::Printer is fully customizable. If you want to change how things are
 displayed, or even its standard behavior, just take a look at the
@@ -339,7 +343,7 @@ or anywhere else.
 
 =over 4
 
-=item * keep your customized settings on a `.dataprinter` file that allows
+=item * keep your custom settings on a `.dataprinter` file that allows
 B<< different options per module >> being analyzed!
 
 =back
@@ -463,14 +467,14 @@ overriden by passing arguments like shown above):
 =head2 The .dataprinter configuration file
 
 The most powerful way to customize Data::Printer is to have a C<.dataprinter>
-file in your home directory, which is a simple I<key = value> text file. It
-lets you set global options to Data::Printer and custom options that will be
-active only on C<p()>/C<np()> calls made inside a given module:
+file in your home directory or your project's root directory. The format
+is super simple and can be understood in the example below:
 
     # global settings (note that only full line comments are accepted)
     max_depth       = 1
     theme           = Monokai
     class.stringify = 0
+
     # use quotes if you want spaces to be significant:
     hash_separator  = " => "
 
@@ -543,6 +547,7 @@ information on each of them:
     max_depth      => 0,
     deparse        => 0,
     alias          => 'p',  # what to call p() when importing
+    warnings       => 1,    # show warnings if something goes wrong
 
     # colorization (see Colors & Themes below)
     colored => 'auto', # 0, 1
@@ -644,7 +649,17 @@ theme, and for a complete list of all natively used color labels.
 
 =head2 Profiles
 
-TBD.
+You may bundle your settings and filters into a profile module.
+It works like a configuration file but gives you the power and flexibility
+to use Perl code to find out what to print and how to print. It also lets
+you use CPAN to store your preferred settings and install them into your
+projects just like a regular dependency.
+
+    use DDP profile => 'ProfileName';
+
+See L<Data::Printer::Profile> for all the ways to load a profile, a list
+of available profiles and how to make one yourself.
+
 
 =head2 Filters
 
