@@ -6,15 +6,7 @@ use Data::Printer::Common;
 sub load_rc_file {
     my ($filename) = @_;
     if (!$filename) {
-        if (exists $ENV{DATAPRINTERRC}) {
-            $filename = $ENV{DATAPRINTERRC};
-        }
-        else {
-            require File::Spec;
-            $filename = File::Spec->catfile(
-                Data::Printer::Common::_my_home, '.dataprinter'
-            );
-        }
+        $filename = _get_first_rc_file_available();
     }
     return unless $filename && -e $filename && !-d $filename;
     if (open my $fh, '<', $filename) {
@@ -26,11 +18,104 @@ sub load_rc_file {
         return _str2data($filename, $rc_data);
     }
     else {
-        Data::Printer::Common::_warn("error opening '$filename': $!");
+        Data::Printer::Common::_warn(undef, "error opening '$filename': $!");
         return;
     }
 }
 
+sub _get_first_rc_file_available {
+    return $ENV{DATAPRINTERRC} if exists $ENV{DATAPRINTERRC};
+
+    # look for a .dataprinter file on the project home up until we reach '/'
+    my $dir = _project_home();
+    require File::Spec;
+    while (defined $dir) {
+        my $file = File::Spec->catfile($dir, '.dataprinter');
+        return $file if -f $file;
+        (my $updir = $dir) =~ s{/[^\/]+/?\z}{/};
+        last if !defined $updir || $updir eq $dir;
+        $dir = $updir;
+    }
+    # still here? look for .dataprinter on the user's HOME:
+    return File::Spec->catfile( _my_home(), '.dataprinter');
+}
+
+sub _my_cwd {
+    require Cwd;
+    my $cwd = Cwd::getcwd();
+    # try harder if we can't access the current dir.
+    $cwd = Cwd::cwd() unless defined $cwd;
+    return $cwd;
+}
+
+sub _project_home {
+    require Cwd;
+    my $path;
+    if ($0 eq '-e' || $0 eq '-') {
+        my $cwd = _my_cwd();
+        $path = Cwd::abs_path($cwd) if defined $cwd;
+    }
+    else {
+        my $script = $0;
+        return unless -f $script;
+        require File::Spec;
+        require File::Basename;
+        # we need the full path if we have chdir'd:
+        $script = File::Spec->catfile(_my_cwd(), $script)
+            unless File::Spec->file_name_is_absolute($script);
+        my (undef, $maybe_path) = File::Basename::fileparse($script);
+        $path = Cwd::abs_path($maybe_path) if defined $maybe_path;
+    }
+    return $path;
+}
+
+# adapted from File::HomeDir && File::HomeDir::Tiny
+sub _my_home {
+    my ($testing) = @_;
+    if ($testing) {
+        require File::Temp;
+        require File::Spec;
+        my $BASE  = File::Temp::tempdir( CLEANUP => 1 );
+        my $home  = File::Spec->catdir( $BASE, 'my_home' );
+        $ENV{HOME} = $home;
+        mkdir($home, 0755) unless -d $home;
+        return $home;
+    }
+    elsif ($^O eq 'MSWin32' and "$]" < 5.016) {
+        return $ENV{HOME} || $ENV{USERPROFILE};
+    }
+    elsif ($^O eq 'MacOS') {
+        my $error = _tryme(sub { require Mac::SystemDirectory; 1 });
+        return Mac::SystemDirectory::HomeDirectory() unless $error;
+    }
+    # this is the most common case, for most breeds of unix, as well as
+    # MSWin32 in more recent perls.
+    my $home = (<~>)[0];
+    return $home if $home;
+
+    # desperate measures that should never be needed.
+    if (exists $ENV{LOGDIR} and $ENV{LOGDIR}) {
+        $home = $ENV{LOGDIR};
+    }
+    if (not $home and exists $ENV{HOME} and $ENV{HOME}) {
+        $home = $ENV{HOME};
+    }
+    # Light desperation on any (Unixish) platform
+    SCOPE: { $home = (getpwuid($<))[7] if not defined $home }
+    if (defined $home and ! -d $home ) {
+        $home = undef;
+    }
+    return $home;
+}
+
+sub _file_mode_is_restricted {
+    my ($filename) = @_;
+    my $mode_raw = (stat($filename))[2];
+    return 0 unless defined $mode_raw;
+    my $mode = sprintf('%04o', $mode_raw & 07777);
+    return (length($mode) == 4 && substr($mode, 2, 2) eq '00') ? 1 : 0;
+}
+
 sub _str2data {
     my ($filename, $content) = @_;
     my $config = { _ => {} };
@@ -44,8 +129,7 @@ sub _str2data {
         if (defined $filter) {
             if ( /^end filter\s*$/ ) {
                 if (!defined $can_use_filters) {
-                    my $mode = sprintf('%04o', (stat(q(/Users/garu/.dataprinter)))[2] & 07777);
-                    $can_use_filters = (length($mode) == 4 && substr($mode, 2, 2) eq '00') ? 1 : 0;
+                    $can_use_filters = _file_mode_is_restricted($filename);
                 }
                 if ($can_use_filters) {
                     my $sub_str = 'sub { my ($obj, $ddp) = @_; '
@@ -55,12 +139,12 @@ sub _str2data {
                     push @{$config->{$ns}{filters}}, +{ $filter->{name} => eval $sub_str };
                 }
                 else {
-                    Data::Printer::Common::_warn("ignored filter '$filter->{name}' from rc file '$filename': file is readable/writeable by others");
+                    Data::Printer::Common::_warn(undef, "ignored filter '$filter->{name}' from rc file '$filename': file is readable/writeable by others");
                 }
                 $filter = undef;
             }
             elsif ( /^begin\s+filter/ ) {
-                Data::Printer::Common::_warn("error reading rc file '$filename' line $counter: found 'begin filter' inside another filter definition ($filter->{name}). Are you missing an 'end filter' on line " . ($counter - 1) . '?');
+                Data::Printer::Common::_warn(undef, "error reading rc file '$filename' line $counter: found 'begin filter' inside another filter definition ($filter->{name}). Are you missing an 'end filter' on line " . ($counter - 1) . '?');
                 return {};
             }
             else {
@@ -72,7 +156,7 @@ sub _str2data {
         }
         elsif ( /^\s*\[\s*(.+?)\s*\]\s*$/ ) {
             # Create the sub-hash if it doesn't exist.
-            # Without this sections without keys will not
+            # Without this, sections without keys will not
             # appear at all in the completed struct.
             $config->{$ns = $1} ||= {};
         }
@@ -110,9 +194,10 @@ sub _str2data {
             $filter = { name => $filter_name, code_str => '' };
         }
         else {
-            Data::Printer::Common::_warn("error reading rc file '$filename': syntax error at line $counter: $_");
-            if ($counter == 1 && /\A\s*{/s) {
+            Data::Printer::Common::_warn(undef, "error reading rc file '$filename': syntax error at line $counter: $_");
+            if ($counter == 1 && /\A\s*\{/s) {
                 Data::Printer::Common::_warn(
+                    undef,
                     "RC file format changed in 1.00. Usually all it takes is:\n"
                   . "cp $filename $filename.old && perl -MData::Printer::Config -E 'say Data::Printer::Config::convert(q($filename))' > $filename\n"
                   . "Please visit https://metacpan.org/pod/Data::Printer::Config for details."
@@ -121,9 +206,73 @@ sub _str2data {
             return {};
         }
     }
+    # now that we have loaded the config, we must expand
+    # all existing 'rc_file' and 'profile' statements and
+    # merge them together.
+    foreach my $ns (keys %$config) {
+        $config->{$ns} = _expand_profile($config->{$ns})
+            if exists $config->{$ns}{profile};
+    }
     return $config;
 }
 
+sub _merge_options {
+    my ($old, $new) = @_;
+    if (ref $new eq 'HASH') {
+        my %merged;
+        my $to_merge = ref $old eq 'HASH' ? $old : {};
+        foreach my $k (keys %$new, keys %$to_merge) {
+            # if the key exists in $new, we recurse into it:
+            if (exists $new->{$k}) {
+                $merged{$k} = _merge_options($to_merge->{$k}, $new->{$k});
+            }
+            else {
+                # otherwise we keep the old version (recursing in case of refs)
+                $merged{$k} = _merge_options(undef, $to_merge->{$k});
+            }
+        }
+        return \%merged;
+    }
+    elsif (ref $new eq 'ARRAY') {
+        # we'll only use the array on $new, but we still need to recurse
+        # in case array elements contain other data structures.
+        my @merged;
+        foreach my $element (@$new) {
+            push @merged, _merge_options(undef, $element);
+        }
+        return \@merged;
+    }
+    else {
+        return $new;
+    }
+}
+
+
+sub _expand_profile {
+    my ($options, $ddp) = @_;
+    my $profile = delete $options->{profile};
+    if ($profile !~ /\A[a-zA-Z0-9:]+\z/) {
+        Data::Printer::Common::_warn($ddp,"invalid profile name '$profile'");
+    }
+    else {
+        my $class = 'Data::Printer::Profile::' . $profile;
+        my $error = Data::Printer::Common::_tryme(sub {
+            my $load_error = Data::Printer::Common::_tryme("use $class; 1;");
+            die $load_error if defined $load_error;
+            my $expanded = $class->profile();
+            die "profile $class did not return a HASH reference" unless ref $expanded eq 'HASH';
+            $options = Data::Printer::Config::_merge_options($expanded, $options);
+        });
+        if (defined $error) {
+            Data::Printer::Common::_warn($ddp, "unable to load profile '$profile': $error");
+        }
+    }
+    return $options;
+}
+
+
+
+
 # converts the old format to the new one
 sub convert {
     my ($filename) = @_;
@@ -164,6 +313,7 @@ sub _convert {
     }
     elsif (ref $value) {
         Data::Printer::Common::_warn(
+            undef,
             " [*] path '$key_str': expected scalar, found " . ref($value)
           . ". Filters must be in their own class now, loaded with 'filter'.\n"
           . "If you absolutely must put custom filters in, use the 'begin filter'"
