@@ -5,7 +5,10 @@ use Data::Printer::Filter;
 use Data::Printer::Common;
 use Scalar::Util ();
 
-filter 'HASH' => sub {
+filter 'HASH' => \&parse;
+
+
+sub parse {
     my ($hash_ref, $ddp) = @_;
     my $tied = '';
     if ($ddp->show_tied and my $tie = ref tied %$hash_ref) {
@@ -36,20 +39,18 @@ filter 'HASH' => sub {
         my $colored_key = Data::Printer::Common::_process_string($ddp, $raw_key, 'hash');
         my $new_key = Data::Printer::Common::_colorstrip($colored_key);
 
-        if ($ddp->quote_keys) {
-            my $needs_quote = 1;
-            if ($ddp->quote_keys eq 'auto') {
-                if ($raw_key eq $new_key && $new_key !~ /\s|\r|\n|\t|\f/) {
-                    $needs_quote = 0;
-                }
-            }
-            if ($needs_quote) {
-                $new_key     = q(') . $new_key . q(');
-                $colored_key = $ddp->maybe_colorize(q('), 'quotes')
-                             . $colored_key
-                             . $ddp->maybe_colorize(q('), 'quotes')
-                             ;
+        if (_needs_quote($ddp, $raw_key, $new_key)) {
+            my $quote_char = $ddp->scalar_quotes;
+            # foo'bar ==> 'foo\'bar'
+            if (index($new_key, $quote_char) >= 0) {
+                $new_key =~ s{$quote_char}{\\$quote_char}g;
+                $colored_key =~ s{$quote_char}{\\$quote_char}g;
             }
+            $new_key     = $quote_char . $new_key . $quote_char;
+            $colored_key = $ddp->maybe_colorize($quote_char, 'quotes')
+                            . $colored_key
+                            . $ddp->maybe_colorize($quote_char, 'quotes')
+                            ;
         }
         $processed_keys{$idx} = {
             raw     => $raw_key,
@@ -75,8 +76,13 @@ filter 'HASH' => sub {
         }
         my $key = $processed_keys{$idx};
 
+        my $original_varname = $ddp->current_name;
         # update 'var' to 'var{key}':
-        $ddp->current_name( $ddp->current_name . '{' . $key->{raw} . '}' );
+        $ddp->current_name(
+            $original_varname
+            . ($ddp->arrows eq 'all' || ($ddp->arrows eq 'first' && $ddp->current_depth == 1) ? '->' : '')
+            . '{' . $key->{nocolor} . '}'
+        );
 
         my $padding = $len - length($key->{nocolor});
         $padding = 0 if $padding < 0;
@@ -90,7 +96,7 @@ filter 'HASH' => sub {
         # a '\' in front of them.
         my $ref = ref $hash_ref->{$key->{raw}};
         if ( $ref && $ref eq 'SCALAR' ) {
-            $string .= $ddp->parse(\\$hash_ref->{ $key->{raw} });
+            $string .= $ddp->parse(\$hash_ref->{ $key->{raw} });
         }
         elsif ( $ref && $ref ne 'REF' ) {
             $string .= $ddp->parse( $hash_ref->{ $key->{raw} });
@@ -102,10 +108,7 @@ filter 'HASH' => sub {
             if $total_keys > 0 || $ddp->end_separator;
 
         # restore var name back to "var"
-        my $size = 2 + length($key->{raw});
-        my $name = $ddp->current_name;
-        substr $name, -$size, $size, '';
-        $ddp->current_name($name);
+        $ddp->current_name($original_varname);
     }
     $ddp->outdent;
     $string .= $ddp->newline . $ddp->maybe_colorize('}', 'brackets');
@@ -116,5 +119,17 @@ filter 'HASH' => sub {
 ### Private auxiliary helpers below ###
 #######################################
 
+sub _needs_quote {
+    my ($ddp, $raw_key, $new_key) = @_;
+    my $quote_keys = $ddp->quote_keys;
+    my $scalar_quotes = $ddp->scalar_quotes;
+    return 0 unless defined $quote_keys && defined $scalar_quotes;;
+    if ($quote_keys eq 'auto'
+        && $raw_key eq $new_key
+        && $new_key !~ /\s|\r|\n|\t|\f/) {
+            return 0;
+    }
+    return 1;
+}
 
 1;
