@@ -68,6 +68,7 @@ my @method_names =qw(
     hash_preserve ignore_keys unicode_charnames colored theme show_weak
     max_depth index separator end_separator class_method class hash_separator
     align_hash sort_keys quote_keys deparse return_value show_dualvar show_tied
+    warnings arrows
 );
 foreach my $method_name (@method_names) {
     no strict 'refs';
@@ -111,9 +112,17 @@ sub _init {
     $self->{'_array_padding'} = 0;
     $self->{'_seen'} = {};
     $self->{_refcount_base} = 3;
+    $self->{'warnings'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'warning', 1);
     $self->{'indent'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'indent', 4);
     $self->{'index'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'index', 1);
     $self->{'name'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'name', 'var');
+    $self->{'arrows'} = Data::Printer::Common::_fetch_anyof(
+        $props,
+        'arrows',
+        'none',
+        [qw(none first all)]
+    );
+
     $self->{'show_tainted'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'show_tainted', 1);
     $self->{'show_tied'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'show_tied', 1);
     $self->{'show_weak'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'show_weak', 1);
@@ -145,7 +154,7 @@ sub _init {
     $self->{'caller_message_newline'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'caller_message_newline', 1);
     $self->{'caller_message_position'} = Data::Printer::Common::_fetch_anyof($props, 'caller_message_position', 'before', [qw(before after)]);
     $self->{'resolve_scalar_refs'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'resolve_scalar_refs', 0);
-    $self->{'string_max'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'string_max', 1024);
+    $self->{'string_max'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'string_max', 4096);
     $self->{'string_preserve'} = Data::Printer::Common::_fetch_anyof(
                              $props,
                              'string_preserve',
@@ -157,7 +166,7 @@ sub _init {
                                 'string_overflow',
                                 '(...skipping __SKIPPED__ chars...)'
                             );
-    $self->{'array_max'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'array_max', 50);
+    $self->{'array_max'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'array_max', 100);
     $self->{'array_preserve'} = Data::Printer::Common::_fetch_anyof(
                              $props,
                              'array_preserve',
@@ -169,7 +178,7 @@ sub _init {
                                 'array_overflow',
                                 '(...skipping __SKIPPED__ items...)'
                         );
-    $self->{'hash_max'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'hash_max', 50);
+    $self->{'hash_max'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'hash_max', 100);
     $self->{'hash_preserve'} = Data::Printer::Common::_fetch_anyof(
                              $props,
                              'hash_preserve',
@@ -279,7 +288,7 @@ sub _load_output_handle {
         $error = 'unknown output data';
     }
     if ($error) {
-        Data::Printer::Common::_warn("error opening custom output handle: $error");
+        Data::Printer::Common::_warn($self, "error opening custom output handle: $error");
         $self->{output_handle} = $targets{'stderr'}
     }
     return;
@@ -363,6 +372,7 @@ sub _load_filters {
     if (exists $props->{filters}) {
         if (ref $props->{filters} eq 'HASH') {
             Data::Printer::Common::_warn(
+                $self,
                 'please update your code: filters => { ... } is now filters => [{ ... }]'
             );
             push @filters, $props->{filters};
@@ -371,7 +381,7 @@ sub _load_filters {
             @filters = @{ $props->{filters} };
         }
         else {
-            Data::Printer::Common::_warn('filters must be an ARRAY reference');
+            Data::Printer::Common::_warn($self, 'filters must be an ARRAY reference');
         }
     }
     foreach my $filter (@filters) {
@@ -383,6 +393,7 @@ sub _load_filters {
             foreach my $k (keys %$filter) {
                 if ($k eq '-external') {
                     Data::Printer::Common::_warn(
+                        $self,
                         'please update your code: '
                       . 'filters => { -external => [qw(Foo Bar)}'
                       . ' is now filters => [qw(Foo Bar)]'
@@ -395,13 +406,14 @@ sub _load_filters {
                 }
                 else {
                     Data::Printer::Common::_warn(
+                        $self,
                         'hash filters must point to a CODE reference'
                     );
                 }
             }
         }
         else {
-            Data::Printer::Common::_warn('filters must be a name or { type => sub {...} }');
+            Data::Printer::Common::_warn($self, 'filters must be a name or { type => sub {...} }');
         }
     }
     return;
@@ -412,7 +424,7 @@ sub _load_external_filter {
     my $module = "Data::Printer::Filter::$class";
     my $error = Data::Printer::Common::_tryme("use $module; 1;");
     if ($error) {
-        Data::Printer::Common::_warn("error loading filter '$class': $error");
+        Data::Printer::Common::_warn($self, "error loading filter '$class': $error");
         return;
     }
     my $from_module = $module->_filter_list;
@@ -488,6 +500,7 @@ sub _load_colors {
         name            => $theme_name,
         color_overrides => $props->{colors},
         color_level     => $self->color_level,
+        ddp             => $self,
     );
     if (!$theme_object) {
         if ($theme_name ne $default_theme) {
@@ -495,6 +508,7 @@ sub _load_colors {
                 name            => $default_theme,
                 color_overrides => $props->{colors},
                 color_level     => $self->color_level,
+                ddp             => $self,
             );
         }
         Data::Printer::Common::_die("Unable to load default theme. This should never happen - please contact the author") unless $theme_object;
@@ -520,18 +534,21 @@ sub _filters_for_data {
     my $ref_kind = Scalar::Util::reftype($data);
     $ref_kind = 'SCALAR' unless $ref_kind;
 
-    # Huh. ref() returns 'Regexp' but reftype() returns 'REGEXP'
+    # ref() returns 'Regexp' but reftype() returns 'REGEXP', so we picked one:
     $ref_kind = 'Regexp' if $ref_kind eq 'REGEXP';
 
     my @potential_filters;
 
     # first, try class name + full inheritance for a specific name.
-    # NOTE: blessed() is returning true for regexes.
-    my $class = $ref_kind eq 'Regexp' ? () : Scalar::Util::blessed($data);
-    # before 5.11 regexes are blessed SCALARs:
-    if ($] < 5.011 && $ref_kind eq 'SCALAR' && defined $class && $class eq 'Regexp') {
-        $ref_kind = 'Regexp';
-        undef $class;
+    my $class = Scalar::Util::blessed($data);
+
+    # a regular regexp is blessed, but in that case we want a
+    # regexp filter, not a class filter.
+    if (defined $class && $class eq 'Regexp') {
+        if ($ref_kind eq 'Regexp' || ($] < 5.011 && $ref_kind eq 'SCALAR')) {
+            $ref_kind = 'Regexp';
+            undef $class;
+        }
     }
     if (defined $class) {
         if ($self->class->parent_filters) {
@@ -573,8 +590,7 @@ sub _filters_for_data {
 sub _see {
     my ($self, $data, %options) = @_;
     return {} unless ref $data;
-
-    my $id = Data::Printer::Common::_object_id($data);
+    my $id = pack 'J', Scalar::Util::refaddr($data);
     if (!exists $self->{_seen}{$id}) {
         $self->{_seen}{$id} = {
             name     => $self->current_name,
@@ -586,11 +602,17 @@ sub _see {
     return $self->{_seen}{$id};
 }
 
+sub seen {
+    my ($self, $data) = @_;
+    my $id = pack 'J', Scalar::Util::refaddr($data);
+    return exists $self->{_seen}{$id};
+}
+
 sub unsee {
     my ($self, $data) = @_;
     return unless ref $data && keys %{$self->{_seen}};
 
-    my $id = Data::Printer::Common::_object_id($data);
+    my $id = pack 'J', Scalar::Util::refaddr($data);
     delete $self->{_seen}{$id};
     return;
 }
@@ -706,11 +728,11 @@ sub _check_memsize {
     });
     if ($error) {
         if ($error =~ m{locate Devel/Size.pm}) {
-            Data::Printer::Common::_warn("Devel::Size not found, show_memsize will be ignored")
+            Data::Printer::Common::_warn($self, "Devel::Size not found, show_memsize will be ignored")
                 if $self->{_position} == 1;
         }
         else {
-            Data::Printer::Common::_warn("error fetching memory usage: $error");
+            Data::Printer::Common::_warn($self, "error fetching memory usage: $error");
         }
         return '';
     }
@@ -726,7 +748,7 @@ sub _check_weak {
     my $realtype = Scalar::Util::reftype($_[0]);
     my $isweak;
     if ($realtype && ($realtype eq 'REF' || $realtype eq 'SCALAR')) {
-        $isweak = Scalar::Util::isweak($_[0] );
+        $isweak = Scalar::Util::isweak($_[0]);
     }
     else {
         $isweak = Scalar::Util::isweak($_[0]);
@@ -845,7 +867,7 @@ the same filter over and over again.
 =item * C<< seen_override => 1 >> - Data::Printer::Object tries to remember
 if it has already seen a data structure before, so it can show the circular
 reference instead of entenring an infinite loop. However, there are cases
-when you wanto to print the same data structure twice, like when you're doing
+when you want to print the same data structure twice, like when you're doing
 a second pass on a blessed object to print its internals, or if you're using
 the same object over and over again. This setting overrides the internal
 counter and prints the same data again. Check L<unsee|/unsee( $data )> below
@@ -919,18 +941,22 @@ Most of them are described in L<Data::Printer>.
 
 =item * align_hash - vertically align hash keys (default: 1)
 
-=item * array_max - maximum array elements to show. Set to 0 to show all (default: 50)
+=item * array_max - maximum array elements to show. Set to 0 to show all (default: 100)
 
 =item * array_overflow - message to display once array_max is reached
 
 =item * array_preserve - which part of the array to preserve after array_max (default: 'begin')
 
+=item * arrows - show ref arrows when displaying circular refs? Can be 'none' (e.g. var{x}[y]), 'first' (e.g. var->{x}[y]) or 'all' (e.g. var->{x}->[y]). Default is 'none'.
+
 =item * caller_info - whether the user wants to prepend dump with caller information or not (default: 0)
 
 =item * caller_message - what to print when caller_info is true.
 
 =item * caller_message_newline - skip line after printing caller_message (default: 1)
 
+=item * caller_message_position - do you want the message displayed 'before' or 'after' the data? (default: 'before')
+
 =item * class - class properties to override.
 
 =item * class_method - function name to look for custom dump of external classes (default: '_dataprinter')
@@ -951,7 +977,7 @@ Most of them are described in L<Data::Printer>.
 
 =item * extra_config - all given options that were not recognized by Data::Printer::Object are kept here. Useful to create custom options in filters. See L<Data::Printer::Filter>.
 
-=item * hash_max - maximum hash pairs to show. Set to 0 to show all (default: 50)
+=item * hash_max - maximum hash pairs to show. Set to 0 to show all (default: 100)
 
 =item * hash_overflow - message to display once hash_max is reached
 
@@ -979,6 +1005,8 @@ Most of them are described in L<Data::Printer>.
 
 =item * quote_keys - whether to quote hash keys or not (default: 'auto')
 
+=item * resolve_scalar_refs - if a reference to a scalar value is found more than once, print the resolved value. Defaults to false.
+
 =item * return_value - whether the user wants the return value to be a pass-through of the source data ('pass'), the dump content itself ('dump') or nothing at all ('void'). Defaults to 'pass'.
 
 =item * scalar_quotes - which quotation character to use when printing strings (default: ")
@@ -1005,7 +1033,7 @@ Most of them are described in L<Data::Printer>.
 
 =item * sort_keys - whether to sort hash keys (default: 1)
 
-=item * string_max - maximum number of characters in a string. Set to 0 to show all (default: 1024)
+=item * string_max - maximum number of characters in a string. Set to 0 to show all (default: 4096)
 
 =item * string_overflow - message to display once string_max is reached
 
@@ -1017,6 +1045,8 @@ Most of them are described in L<Data::Printer>.
 
 =item * write_label - returns the proper label string, as parsed from caller_message.
 
+=item * warnings - show warning messages if something goes wrong.
+
 =back
 
 =head1 SEE ALSO
